
/* receiver.c -- user-space part, receive the three signals for button,
	deasil, widdershins, and react to them. */ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "gammapb_u.h"

/* Write some text to the gammapb device */ 
int GPBU_Send(char *txt)
{
	int fd, ws;
	fd = open( DEVICENAME, O_WRONLY); 
	if(fd == -1)
	{
		return(-1);
	}

	ws = write(fd, txt, strlen(txt));
	close(fd);
	return(ws);
}


/* Turn the signals on for us. */ 
int GPBU_TurnOn()
{
	int ws; 
	char txt[80];
	snprintf(txt, 80, "P%u\n", getpid()); 
	ws = GPBU_Send(txt);
	return(ws);
}

/* Turn the signals on for us. */ 
int GPBU_TurnOff()
{
	int ws; 
	ws = GPBU_Send("P0");
	return(ws);
}


static void on_push(int sig, siginfo_t *siginfo, void *c)
{
	printf("Got a push\n");
	usleep(100);
	// GPBU_TurnOn();
	return;
}

static void on_deasil(int sig, siginfo_t *siginfo, void *c)
{
	printf("Noted deasil movement\n");
	usleep(100);
	// GPBU_TurnOn();
	return;
}

static void on_widdershins(int sig, siginfo_t *siginfo, void *c)
{
	printf("Noted widdershins movement\n");
	usleep(100);
	// GPBU_TurnOn();
	return;
}


int G_rflag;

void sigintcatcher(int sig, siginfo_t *siginfo, void *c)
{
	printf("Caught SIGINT, will exit.\n" ); 
	G_rflag = 1;
	return;
}

int main(int argc, char **argv)
{
	struct sigaction sact;

	memset(&sact, 0, sizeof(struct sigaction)); 
	sact.sa_sigaction = sigintcatcher;
	sact.sa_flags = SA_SIGINFO | SA_RESTART; 
	sigaction(SIGINT, &sact, 0);

	memset(&sact, 0, sizeof(struct sigaction)); 
	sact.sa_sigaction = on_push;
	sact.sa_flags = SA_SIGINFO | SA_RESTART; 
	sigaction(SIG_PB, &sact, 0);

	memset(&sact, 0, sizeof(struct sigaction)); 
	sact.sa_sigaction = on_deasil;
	sact.sa_flags = SA_SIGINFO | SA_RESTART; 
	sigaction(SIG_DEASIL, &sact, 0);

	memset(&sact, 0, sizeof(struct sigaction)); 
	sact.sa_sigaction = on_widdershins;
	sact.sa_flags = SA_SIGINFO | SA_RESTART; 
	sigaction(SIG_WIDDERSHINS, &sact, 0);

	G_rflag = 0;

	GPBU_TurnOn();

	for(;;)
	{
		/* In the loop we can look at the signals having been sent. */

		if(G_rflag) break;
	}

	GPBU_Send("stop");
	GPBU_TurnOff();
	return(0);
}



