

/* gon.c -- continue going straight in whatever direction 
	we may be facing, for some number of wheel pulses. */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdint.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include <poll.h>
#include <signal.h>
#include <math.h>

#include <utilib.h>
#include "hmc6352.h"
#include "mgu.h"

// int go_straight(int ireq);

int main(int argc, char **argv)
{
	double hh;
	int ireq, odo, lco, rco;
	char *stop;

	/* How many ticks should we run? */ 
	ireq = 0;
	if(argc >= 2)
		ireq = strtol(argv[1], &stop, 0); 

	hh = anorm(rdhd());
	printf("Heading = %G\n", hh); 

	odo = go_straight(ireq, &lco, &rco);
	printf("Moved L=%d R=%d S=%d\n", lco, rco, odo); 

	hh = anorm(rdhd());
	printf("Heading = %G\n", hh); 

	return(0);
}


#if(0)

int go_straight(int ireq)
{

	/* Logic:
		move L and R wheel, making sure they generate the 
		same number of pulses. Never mind which direction 
		we are facing. */ 
	
	struct pollfd fdset[5];
	int leftfd, rightfd; 
	int rc;
	char rdbuf[20];
	int c;
	int odo;
	int lco, rco;
	int nreq;

	if(ireq == 0) return(0); 

	if(ireq < 0)
	{
		nreq = -ireq;
		setgpio(RDIR, GBACK);
		setgpio(LDIR, GBACK);
	}
	else
	{
		nreq = ireq;
		setgpio(RDIR, GFWD);
		setgpio(LDIR, GFWD);
	}

	lco = rco = odo = 0; 

	leftfd = open("/sys/class/gpio/gpio8/value", O_RDONLY | O_NONBLOCK); 
	rightfd = open("/sys/class/gpio/gpio7/value", O_RDONLY | O_NONBLOCK); 

	/* loop */
	for(;;)
	{
		memset( (void *) fdset, 0, 3 * sizeof(struct pollfd));
		fdset[0].fd = leftfd;
		fdset[0].events = POLLPRI;
		fdset[0].revents = 0;
		fdset[1].fd = rightfd;
		fdset[1].events = POLLPRI;
		fdset[1].revents = 0;
		fdset[2].fd = fileno(stdin);
		fdset[2].events = POLLIN;
		fdset[2].revents = 0;

		rc = poll(fdset, 3, 10);

		if(rc < 0 && errno != 4)
		{
			printf("%s(%d): Poll error %d\n", __FILE__, __LINE__, errno);
			break;
		}


		if(rc == 0)
		{
			/* printf("%s(%d): Time-check: Rq=%d Odo=%d L=%d R=%d\n", 
				__FILE__, __LINE__, nreq*2, odo, lco, rco ); */ 

			if(odo > nreq*2) break; 

			if(lco < rco) { setgpio(LGO, 1); }
			else if(lco > rco) { setgpio(RGO, 1); }
			else 
			{
				setgpio(LGO, 1);
				setgpio(RGO, 1);
			}
			continue;
		}

		/* Left wheel moved */
		if(fdset[0].revents & POLLPRI)
		{
			read(fdset[0].fd, rdbuf, 2); 
			lco ++;
			odo ++;
			/* printf("%s(%d): Left Moved now L=%d Odo=%d\n", 
				__FILE__, __LINE__, lco, odo ); */ 
			setgpio(LGO, 0);
		}

		/* right wheel moved */
		if(fdset[1].revents & POLLPRI)
		{
			read(fdset[1].fd, rdbuf, 2); 
			c = rdbuf[0];
			rco ++;
			odo ++;
			/* printf("%s(%d): Right Moved now R=%d Odo=%d\n", 
				__FILE__, __LINE__, rco, odo ); */ 
			setgpio(RGO, 0);
		}

		/* stdin */
		if(fdset[2].revents & POLLIN)
		{
			read(fdset[2].fd, rdbuf, 1); 
			c = rdbuf[0];
			/* printf("%s(%d): stdin hit %d %c\n", __FILE__, __LINE__, c,  c); */ 
			if('q' == c) break;
			if('Q' == c) break;
		}
	}

	setgpio(LGO, 0);
	setgpio(RGO, 0);

	close(leftfd);
	close(rightfd);

	return(odo);
}

#endif


#if(0)


#define LGO 17
#define LDIR 4
#define RGO 10
#define RDIR 25
#define GFWD 1
#define GBACK 0


void mimpLRs(int durL, int durR)
{
	int durmin, ddu, stop1, stop2;
	int aimL, aimR, adurL, adurR;

	adurL = abs(durL);
	adurR = abs(durR);
	aimL = 1; if(durL < 0) aimL = 0; 
	aimR = 1; if(durR < 0) aimR = 0; 

	if(adurL < adurR)
	{
		durmin = adurL;
		ddu = adurR-adurL;	
		stop1 = 17; /* Left stops first */
		stop2 = 10;
	}
	else
	{
		durmin = adurR;
		ddu = adurL-adurR;	
		stop1 = 10;
		stop2 = 17; /* Left stops last */
	}

	setgpio(4, aimL); 
	setgpio(25, aimR); 
	setgpio(17, 1);
	setgpio(10, 1);

	delay(durmin); 
	setgpio(stop1, 0);
	if(ddu) delay(ddu);
	setgpio(stop2, 0);
}

#endif






