
/* hmc6352.c -- read the heading of the
	HMC6352 at ba 0x21, send it an A 
	and read the returned 16 bits. */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <utilib.h>
#include <linux/i2c-dev.h>
#include "hmc6352.h"

/* Read and return the heading (0-359, -1 on errors)*/
double rdhd()
{
	char i2cdev[128];
	int ba, fd, s;
	unsigned int hdh, hdl, hdx;
	double hdd;

	ba = 0x21;
	strcpy(i2cdev, "/dev/i2c-1"); 

	fd = open(i2cdev, O_RDWR);
	if(fd == -1) 
	{
		perror(i2cdev); 
		return(-1.0); 
	}

	s = ioctl(fd, I2C_SLAVE, ba);
	if(s == -1) 
	{
		fprintf(stderr, "Bus address %d ", ba); 
		perror("I2C_SLAVE");
		close(fd);
		return(-1.0);
	}

	s = i2c_smbus_write_byte(fd, 0x41);
		/* Command A, will return heading */
	
	if(s == -1)
	{
		fprintf(stderr, "Bus address %d ", ba); 
		perror("write byte");
	}

	delay(10);
	
	hdx = i2c_smbus_read_word_data(fd, 0x41);  
	// fprintf(stderr, "Read heading 0x%x\n", hdx); 
	hdl = (hdx & 0xFF00) >> 8;
	hdh = hdx & 0xFF;

	// hdl = i2c_smbus_read_byte(fd);  
	// hdh = i2c_smbus_read_byte(fd);  
	hdx = hdl + (hdh << 8);
	
	// fprintf(stderr, "Read heading 0x%x\n", hdx); 
	hdd = hdx / 10.0;

	/* fprintf(stderr, "Read heading %.1f\n", hdd);  */ 

	close(fd);
	return(hdd);
}

/* Make sure some angle always is between -180 and +180 */ 
double anorm(double ai)
{
	if(ai < -180.0) 
	{
		while(ai < -180.0) ai += 360.0;
		return(ai);
	}

	if(ai >  180.0) 
	{
		while(ai > 180.0) ai -= 360.0;
		return(ai);
	}

	return(ai);
}




