
/* odopro.c -- odometry process: */

/* run in the background, poll/read the wheel 
	sensors and put	counter values into shared memory 
	Also handles the controls of the L and R motors.
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdint.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include <poll.h>
#include <signal.h>
#include <math.h>

#include <sys/select.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include <sys/shm.h>

// #include <utilib.h>
// #include "hmc6352.h"
#include "mgu.h"

#include "sem1.h"
#include "odopro.h"


#if(0)
	sem_P(semid, SEMSHMX);
	odoshm = (struct spirou_odo *) shmat(shmid, NULL, SHM_READONLY);
	X = odoshm->Lco;
	X = odoshm->Rco;
	X = odoshm->odo;
	shmdt(odoshm);
	sem_V(semid, SEMSHMX);

	sem_P(semid, SEMSHMX);
	odoshm = (struct spirou_odo *) shmat(shmid, NULL, 0);
	odoshm->Lco ++;
	odoshm->Rco ++;
	odoshm->odo ++;
	shmdt(odoshm);
	sem_V(semid, SEMSHMX);
#endif


int G_rflag;

void sigintcatcher(int sig, siginfo_t *sinfo, void *vpx)
{
    fprintf(stderr, "%s(%d): Caught SIGINT, will exit...\n", 
		__FILE__, __LINE__);
    G_rflag = 1;
    return;
}


int main(int argc, char **argv)
{
	int semid, shmid;
	struct spirou_odo *odoshm;
	struct pollfd fdset[5];
	int leftfd, rightfd; 
	char rdbuf[3];
	int rc;
	struct sigaction sact;

	G_rflag = 0;
	memset(&sact, 0, sizeof(struct sigaction));
	sact.sa_sigaction = sigintcatcher;
	sact.sa_flags = SA_SIGINFO | SA_RESTART;
	sigaction(SIGINT, &sact, 0); 

	semid = sem_create(SPIROUKEY, SEMACOUNT);
	shmid = shmget(SPIROUKEY, sizeof(struct spirou_odo), IPC_CREAT | 0666); 

	sem_P(semid, SEMSHMX);
	odoshm = (struct spirou_odo *) shmat(shmid, NULL, 0);
	odoshm->Ldir = GFWD;
	odoshm->Rdir = GFWD;
	odoshm->Lthr = 0; 		/* Thr 0 will mean stop or standstill. */
	odoshm->Rthr = 0;
	shmdt(odoshm);
	sem_V(semid, SEMSHMX);

	/* Z for zero. */
	if(argc >= 2 && !strcmp(argv[1], "z"))
	{
		sem_P(semid, SEMSHMX);
		odoshm = (struct spirou_odo *) shmat(shmid, NULL, 0);
		odoshm->Lco = 0;
		odoshm->Rco = 0;
		odoshm->odo = 0;
		shmdt(odoshm);
		sem_V(semid, SEMSHMX);
	}

	leftfd = open("/sys/class/gpio/gpio8/value", O_RDONLY | O_NONBLOCK); 
	rightfd = open("/sys/class/gpio/gpio7/value", O_RDONLY | O_NONBLOCK); 

	while(G_rflag)
	{
		memset( (void *) fdset, 0, 3 * sizeof(struct pollfd));
		fdset[0].fd = leftfd;
		fdset[0].events = POLLPRI;
		fdset[0].revents = 0;
		fdset[1].fd = rightfd;
		fdset[1].events = POLLPRI;
		fdset[1].revents = 0;

		rc = poll(fdset, 2, 10);

		if(rc < 0 && errno != 4)
		{
			printf("%s(%d): Poll error %d\n", __FILE__, __LINE__, errno);
			break;
		}

		/* time-out. */ 
		if(rc == 0) 
		{ 
			sem_P(semid, SEMSHMX);
			odoshm = (struct spirou_odo *) shmat(shmid, NULL, 0);

			setgpio(LDIR, odoshm->Ldir);
			setgpio(RDIR, odoshm->Rdir);

			/* If left is required to move, turn it on */
			if(odoshm->Lco < odoshm->Lthr) { setgpio(LGO,1); }
			else                           { setgpio(LGO, 0); }

			/* If right is required to move, turn it on */
			if(odoshm->Rco < odoshm->Rthr) { setgpio(RGO,1); }
			else                           { setgpio(RGO, 0); }

			shmdt(odoshm);
			sem_V(semid, SEMSHMX);

			continue; 
		}

		/* Left wheel moved */
		if(fdset[0].revents & POLLPRI)
		{
			read(fdset[0].fd, rdbuf, 2); 
			sem_P(semid, SEMSHMX);
			odoshm = (struct spirou_odo *) shmat(shmid, NULL, 0);
			odoshm->Lco ++;
			odoshm->odo ++;
			shmdt(odoshm);
			sem_V(semid, SEMSHMX);
			setgpio(LGO, 0);
		}

		/* right wheel moved */
		if(fdset[1].revents & POLLPRI)
		{
			read(fdset[1].fd, rdbuf, 2); 
			sem_P(semid, SEMSHMX);
			odoshm = (struct spirou_odo *) shmat(shmid, NULL, 0);
			odoshm->Rco ++;
			odoshm->odo ++;
			shmdt(odoshm);
			sem_V(semid, SEMSHMX);
			setgpio(RGO, 0);
		}
	}

	close(leftfd);
	close(rightfd);

	return(0);
}




