
/* kterm.c -- Terminal. */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <utilib.h>
#include <fcntl.h>

/* From kbhit.c */ 
int rawin();
int norawin();
int kbhit();
int getch();

/* Read and write stuff to the
	/dev/stn41dev
*/

int main(int argc, char **argv) 
{
    int stopc, i, s, k;
	int fd;
	char txb[4];
	int nr, nw;
  
    char logfil[90];
    FILE *fp;
    char commdev[90];
    int quietquit, nobell, stufnl; 

	snprintf(commdev, 90, "%s", "/dev/stn41dev");

    int kflag = 0;    /* -K */ 
    int fakeflag = 0; /* -f */ 
    int noflush = 0;  /* -F */ 
    quietquit = 0;    /* -q */
    nobell = 1;       /* -B */
    stufnl = 0;       /* -L */ 

    fp = NULL;

    if(scf("coFfqBK", "co", &argc, &argv) == 0)
    {
        fprintf(stderr , 
"usage: %s -c device -o logfile\n\
        -f -F -q -K\n\
where:\n\
    -c device    = use that device rather than %s\n\
    -o logfile   = put all received input into this logfile\n\
    -f           = fake send CRs from Control-F\n\
    -F           = Do not fflush(stdout)\n\
    -q           = Do not include preamble with -o file\n\
    -B           = Do not suppress Beeps to stdout (BEL, ASCII 7)\n\
    -K           = print control-characters directly instead of \n\
                   transcribing them as for example ^G for BEL\n\
", argv[0], commdev);

        exit(1);
    }

    for(i = 1; strcmp("--", argv[i]); i++)
    {
        if(!strcmp("-c", argv[i]))
        {
            i++;
            snprintf(commdev, 90, "%s", argv[i]);
        }
        else
        if(!strcmp("-o", argv[i]))
        {
            i++;
            strcpy(logfil, argv[i]);

            fp = fopen(logfil, "w");

            if(fp == NULL)
            {
                perror(logfil);
            }
        }
        else
        if(!strcmp("-f", argv[i]))
        {
            fakeflag = 1;
        }
        else
        if(!strcmp("-F", argv[i]))
        {
            noflush = 1;
        }
        else
        if(!strcmp("-q", argv[i]))
        {
            quietquit = 1;    /* -q */
        }
        else
        if(!strcmp("-B", argv[i]))
        {
            nobell = 0;       /* -B */
        }
        else
        if(!strcmp("-K", argv[i]))
        {
            kflag = 1;       /* -K */ 
        }
    }
    scfend();

    stopc = 5;

	fprintf(stderr, "TERM41 talk to device %s\n", commdev);

    if(!quietquit)
    {
        if(fp)
            fprintf(fp, "TERM41 talk to device %s\n", commdev);
    }

    /* Main loop for dumb terminal version */
	fd = open(commdev, O_RDWR | O_NONBLOCK);
	if(fd == -1) { perror(commdev); exit(2); }

    k = stopc-1;
    fprintf(stderr, "Enter ^%c to stop\n", '@'+stopc);

    rawin();

    do
    {
		nr = read(fd, txb, 1); 
		if(nr == 1)
		{
            s = txb[0];

            // printf("Read %c ", s); 
	
			if(kflag)
			{
				if(! (nobell && s == 7))
				{
					fputc(s, stdout);
				}
			}
			else
			{
				/* Translate; retain only 
					carriage-return 13, newline 10, backspace 8, 
					and delete 127 */ 

				if(s == 8 || s == 10 || s == 13 || s >= 32) 
				{
					fputc(s, stdout);
				}
				else
				{
					/* Put out control chars in red */ 
					printf("\033[1;31m^%c\033[0;39m", s + 0x40); 
					    /* "\033[1;31m" - Red  */ 
					    /* "\033[1;32m" - Green */ 
					    /* "\033[1;34m" - Blue */ 
					    /* "\033[0;39m" - Standard bright */ 
				}
            }

            if(stufnl && s == 13) fputc(10, stdout);

            if(!noflush) 
                fflush(stdout); 

            if(fp)
            {
                fputc(s, fp);
                fflush(fp);
                if(stufnl && s == 13) fputc(10, fp);
            }
        }

        if(kbhit())
        {
			k = getch();
			if(k != stopc)
			{
				nw = 1;
				txb[0] = k;
				if(k == 6 && fakeflag)
				{
					txb[1] = 13;
					nw = 2; 
				}
				write(fd, txb, nw); 
			}
        }
    }
    while(k != stopc);

    norawin();

	close(fd);

    if(fp) 
      fclose(fp);

    return(0);
}

#if(0)


/* Using the info from the configuration file. set default port values. */

int inidef(char *port, int *baud, int *dbits, int *sbits) 
{
    char txt[90];

    int fcfg;
    char inifile[90];

    fcfg = locatecfg(".kcpcfg", inifile);
    printf("Config file %s %d\n", inifile, fcfg);

    if(!fcfg)
    {
        fprintf(stderr, "No .kcpcfg file found anywhere. Will use defaults.\n" );
        strcpy(port, "/dev/ttyS0");
        *baud = 9600;
        *dbits = 8;
        *sbits = 1;
        return(0);
    }
    /* printf("Prior to GetPrivateProfileString()\n" ); */

    GetPrivateProfileString("KTERM", "C", "/dev/ttyS0", port, 90, inifile);
    /* printf("P string %s for %s\n", port, "C" ); */

    GetPrivateProfileString("KTERM", "B", "9600", txt, 90, inifile);
    /* printf("P string %s for %s\n", txt, "B" );  */
    *baud = (int) argval(txt, 1L, 115200L);

    GetPrivateProfileString("KTERM", "D", "8", txt, 90, inifile);
    /* printf("P string %s for %s\n", txt, "D" ); */
    *dbits = (int) argval(txt, 5L, 8L);

    GetPrivateProfileString("KTERM", "S", "1", txt, 90, inifile);
    /* printf("P string %s for %s\n", txt, "S" );  */
    *sbits = (int) argval(txt, 1L, 2L);
    return(0);
}




int main(int argc, char **argv) 
{
    int stopc, i, s, k;
    int sps;
    void *h com;
  
    int br, db, sb;
    char logfil[90];
    FILE *fp;
    char commdev[90];
    int srts, sdtr; 
    int stripbit;
    int quietquit, nobell, stufnl; 

    int kflag = 0;    /* -K */ 
    int fakeflag = 0; /* -f */ 
    int noflush = 0;  /* -F */ 
    srts = 0;         /* -R x */ 
    sdtr = 0;         /* -D x */ 
    stripbit = 0;     /* -7 */ 
    quietquit = 0;    /* -q */
    nobell = 1;       /* -B */
    stufnl = 0;       /* -L */ 

    inidef(commdev, &br, &db, &sb);

    fp = NULL;

    if(scf("cbdsoFfqBL7SMK", "cbdsoSM", &argc, &argv) == 0)
    {
        fprintf(stderr , 
"usage: %s -c device -b baudrate -d databits -s stopbits -o logfile\n\
        -f -F -q -B -L -7 -S R -S D -M R -M D -7 -K\n\
where:\n\
    -c device    = use that device rather than %s\n\
    -b rate      = use that rate rather than %d\n\
    -d 7         = use 7 data bits (default is %d)\n\
    -s 1 or -s 2 = for 1 or 2 stop bits\n\
    -o logfile   = put all received input into this logfile\n\
    -f           = fake send CRs from Control-F\n\
    -F           = Do not fflush(stdout)\n\
    -q           = Do not include preamble with -o file\n\
    -B           = Do not suppress Beeps to stdout (BEL, ASCII 7)\n\
    -L           = Add newline (LF, ASCII 10) after (CR, ASCII 13) \n\
                    when this is transmitted or received. \n\
    -7           = Strip high bit from read data. \n\
    -S R         = Set RTS (4/25 or 7/9) to Space (positive)\n\
    -S D         = Set DTR (20/25 or 4/9) to Space (positive)\n\
    -M R         = Set RTS (4/25 or 7/9) to Mark (negative)\n\
    -M D         = Set DTR (20/25 or 4/9) to Mark (negative)\n\
    -K           = print control-characters directly instead of \n\
                   transcribing them as for example ^G for BEL\n\
", argv[0], commdev, br, db);

    /* and quit after transmitting with -i */ 

    /* RTS is pin 4/25 or 7/9 */ 
    /* DTR is pin 20/25 or 4/9 */ 

        exit(1);
    }

    for(i = 1; strcmp("--", argv[i]); i++)
    {
        if(!strcmp("-c", argv[i]))
        {
            i++;
            strcpy(commdev, argv[i]);
        }
        else
        if(!strcmp("-b", argv[i]))
        {
            i++;
            br = argval(argv[i], 300, 115200);
        }
        else
        if(!strcmp("-d", argv[i]))
        {
            i++;
            db = argval(argv[i], 5, 8);
        }
        else
        if(!strcmp("-s", argv[i]))
        {
            i++;
            sb = argval(argv[i], 1, 2);
        }
        else
        if(!strcmp("-S", argv[i]))
        {
            i++;
            if((*argv[i]) == 'R')  srts = 1; 
            if((*argv[i]) == 'D')  sdtr = 1; 
        }
        else
        if(!strcmp("-M", argv[i]))
        {
            i++;
            if((*argv[i]) == 'R')  srts = -1; 
            if((*argv[i]) == 'D')  sdtr = -1; 
        }
        else
        if(!strcmp("-o", argv[i]))
        {
            i++;
            strcpy(logfil, argv[i]);

            fp = fopen(logfil, "w");

            if(fp == NULL)
            {
                perror(logfil);
            }
        }
        else
        if(!strcmp("-f", argv[i]))
        {
            fakeflag = 1;
        }
        else
        if(!strcmp("-F", argv[i]))
        {
            noflush = 1;
        }
        else
        if(!strcmp("-7", argv[i]))
        {
            stripbit = 1;
        }
        else
        if(!strcmp("-q", argv[i]))
        {
            quietquit = 1;    /* -q */
        }
        else
        if(!strcmp("-B", argv[i]))
        {
            nobell = 0;       /* -B */
        }
        else
        if(!strcmp("-L", argv[i]))
        {
            stufnl = 1;       /* -L */ 
        }
        else
        if(!strcmp("-K", argv[i]))
        {
            kflag = 1;       /* -K */ 
        }
    }
    scfend();

    stopc = 5;

    fprintf(stderr, "KTERM Serial setting: %s: %d,n,%d,%d\n", commdev, br, db, sb);

    if(!quietquit)
    {
        if(fp)
            fprintf(fp, "KTERM Serial setting: %s: %d,n,%d,%d\n", commdev, br, db, sb);
    }

    /* Main loop for dumb terminal version */

    h com = startserial1(commdev);
    if(h com == NULL) { perror(commdev); exit(2); }

    sps = setpara1(h com, br, db, sb);
    if(sps == -1) { perror(commdev); exit(2); }

    /* Set RTS or DTR accordingly */ 
    if(srts)
    {
        if(srts == -1) srts = 0; 
        AssertRTS1(h com, srts);
    }

    if(sdtr)
    {
        if(sdtr == -1) sdtr = 0; 
        AssertDTR1(h com, sdtr);
    }   

    k = stopc-1;

    fprintf(stderr, "Enter ^%c to stop\n", '@'+stopc);

#if(0)
    if(quietquit)
    {
        if(fp) 
            fprintf(fp, "Enter ^%c to stop\n", '@'+stopc); */
    }
#endif

    rawin();

    do
    {
        s = serpeekc1(h com);

        if(s != -1)
        {
            s = sergetc1(h com);
            /* printf("Read %c ", s, stdout); */
            if(stripbit) { s &= 0x7F; } 
	
			if(kflag)
			{
				if(! (nobell && s == 7))
				{
					fputc(s, stdout);
				}
			}
			else
			{
				/* Translate; retain only 
					carriage-return 13, newline 10, backspace 8, 
					and delete 127 */ 

				if(s == 8 || s == 10 || s == 13 || s >= 32) 
				{
					fputc(s, stdout);
				}
				else
				{
					/* Put out control chars in red */ 
					printf("\033[1;31m^%c\033[0;39m", s + 0x40); 
					    /* "\033[1;31m" - Red  */ 
					    /* "\033[1;32m" - Green */ 
					    /* "\033[1;34m" - Blue */ 
					    /* "\033[0;39m" - Standard bright */ 
				}
            }

            if(stufnl && s == 13) fputc(10, stdout);

            if(!noflush) 
                fflush(stdout); 

            if(fp)
            {
                fputc(s, fp);
                fflush(fp);
                if(stufnl && s == 13) fputc(10, fp);
            }
        }

        if(kbhit())
        {
            k = getch();
            if(k != stopc)
                serputc1(k, h com);
            if(k == 6 && fakeflag)
                serputc1(13, h com);
        }

    }
    while(k != stopc);

    norawin();

    stopserial1(h com);

    if(fp) 
      fclose(fp);

    return(0);
}

#endif


